class Nagios_Reporter
  attr_reader :state

  def initialize
    # 2d array of reports implemented with a hash
    @reports = Hash.new
    @type = :all
    @silent_exit = false
    @state = -1
  end
  def set_type(type=:all)
    if type == :all or type == :worst_only
      @type = type
    else
      unknown!("Invalid reporter type, #{type}, use :all or :worst_only")
    end
  end
  def silence
    @silent_exit = true
  end
  def silence!
    @silent_exit = true
    final_report
  end
  def ok(message)
    report(0,message)
  end
  def ok!(message)
    report!(0,message)
  end
  def warning(message)
    report(1,message)
  end
  def warning!(message)
    report!(1,message)
  end
  def critical(message)
    report(2,message)
  end
  def critical!(message)
    report!(2,message)
  end
  def unknown(message)
    report(3,message)
  end
  def unknown!(message)
    report!(3,message)
  end
  def state_header(state)
    message_header = {0 => "OK: ", 1 => "WARNING: ", 2 => "CRITICAL: ", 3 => "UNKNOWN: "}
    message_header[state]
  end
  def final_report
    message = ""
    if @type == :all
      state_spacer = ""
      @reports.keys.sort.reverse.each do |state|
        message << state_spacer
        message << state_header(state)
        message << @reports[state].join(", ")
        state_spacer = ". "
      end
    else
      message << state_header(state)
      message << @reports[@state].join(", ")
    end

    @state = 3 if @state == -1
    message = "Check gathered no information" if message == ""

    exit @state if @silent_exit
    report!(@state,message,true)
  end
  def report!(state,message,no_state_header=false)
    # report with passed state and message immediately then exit
    print state_header(state) unless no_state_header
    puts message
    exit state
  end
  def report(state,message)
    # update global state if new state is higher
    @state = @state > state ? @state : state
    # add to list of reports of given state
    @reports[state] ||= []
    @reports[state].push(message)
  end
end
